@@ -18,6 +18,7 @@
 #include <QTableView>
 #include <QTableWidget>
 #include <QtWidgets/QColorDialog>
+#include "obs-websocket-api.h"
 
 OBS_DECLARE_MODULE()
 OBS_MODULE_AUTHOR("Exeldro");
@@ -69,31 +70,22 @@ static void load_transition_matrix(obs_data_t *obj)
 
 static void load_transitions(obs_data_t *obj)
 {
-	obs_data_array_t *transitions =
-		obs_data_get_array(obj, "transitions");
+	obs_data_array_t *transitions = obs_data_get_array(obj, "transitions");
 	if (!transitions)
 		return;
-	const size_t count =
-		obs_data_array_count(transitions);
+	const size_t count = obs_data_array_count(transitions);
 	for (size_t i = 0; i < count; i++) {
-		obs_data_t *transition =
-			obs_data_array_item(transitions,
-			                    i);
-		string fromScene = obs_data_get_string(
-			transition, "from_scene");
-		string toScene = obs_data_get_string(
-			transition, "to_scene");
+		obs_data_t *transition = obs_data_array_item(transitions, i);
+		string fromScene =
+			obs_data_get_string(transition, "from_scene");
+		string toScene = obs_data_get_string(transition, "to_scene");
 		string transitionName =
-			obs_data_get_string(
-				transition,
-				"transition");
+			obs_data_get_string(transition, "transition");
 		const uint32_t duration =
-			obs_data_get_int(transition,
-			                 "duration");
-		transition_table[fromScene][toScene]
-			.transition = transitionName;
-		transition_table[fromScene][toScene]
-			.duration = duration;
+			obs_data_get_int(transition, "duration");
+		transition_table[fromScene][toScene].transition =
+			transitionName;
+		transition_table[fromScene][toScene].duration = duration;
 		obs_data_release(transition);
 	}
 	obs_data_array_release(transitions);
@@ -210,8 +202,11 @@ static void clear_transition_overrides()
 static void set_transition_overrides()
 {
 	obs_source_t *scene = obs_frontend_get_current_scene();
-	string fromScene = obs_source_get_name(scene);
-	obs_source_release(scene);
+	string fromScene;
+	if (scene) {
+		fromScene = obs_source_get_name(scene);
+		obs_source_release(scene);
+	}
 
 	auto fs_it = fromScene.empty() ? transition_table.end()
 				       : transition_table.find(fromScene);
@@ -317,6 +312,53 @@ bool disable_hotkey(void *data, obs_hotkey_pair_id id, obs_hotkey_t *hotkey,
 	return false;
 }
 
+static void get_transition(std::string from_scene, std::string to_scene,
+			   std::string &transition, int &duration)
+{
+	auto fs_it = transition_table.find(from_scene);
+	auto as_it = transition_table.find("Any");
+	if (fs_it != transition_table.end()) {
+		auto to_it = fs_it->second.find(to_scene);
+		if (to_it == fs_it->second.end()) {
+			to_it = fs_it->second.find("Any");
+		}
+		if (to_it != fs_it->second.end()) {
+			transition = to_it->second.transition;
+			duration = to_it->second.duration;
+		}
+	}
+	if (transition.empty() && as_it != transition_table.end()) {
+		auto to_it = as_it->second.find(to_scene);
+		if (to_it == as_it->second.end()) {
+			to_it = as_it->second.find("Any");
+		}
+		if (to_it != as_it->second.end()) {
+			transition = to_it->second.transition;
+			duration = to_it->second.duration;
+		}
+	}
+}
+
+static void proc_get_transition(void *data, calldata_t *cd)
+{
+	UNUSED_PARAMETER(data);
+	const char *val = nullptr;
+	std::string from_scene;
+	if (calldata_get_string(cd, "from_scene", &val) && val) {
+		from_scene = val;
+	}
+	val = nullptr;
+	std::string to_scene;
+	if (calldata_get_string(cd, "to_scene", &val) && val) {
+		to_scene = val;
+	}
+	string transition;
+	auto duration = 0;
+	get_transition(from_scene, to_scene, transition, duration);
+	calldata_set_string(cd, "transition", transition.c_str());
+	calldata_set_int(cd, "duration", duration);
+}
+
 bool obs_module_load(void)
 {
 	blog(LOG_INFO, "[Transition Table] loaded version %s", PROJECT_VERSION);
@@ -346,10 +388,117 @@ bool obs_module_load(void)
 		"transition-table.disable",
 		obs_module_text("TransitionTable.Disable"), enable_hotkey,
 		disable_hotkey, nullptr, nullptr);
-
+	auto ph = obs_get_proc_handler();
+	proc_handler_add(
+		ph,
+		"void get_transition_table_transition(string from_scene, string to_scene, out string transition, out int duration)",
+		proc_get_transition, nullptr);
 	return true;
 }
 
+static obs_websocket_vendor vendor = nullptr;
+
+static void vendor_get_transition(obs_data_t *request_data,
+				  obs_data_t *response_data, void *param)
+{
+	UNUSED_PARAMETER(param);
+	std::string from_scene =
+		obs_data_get_string(request_data, "from_scene");
+	std::string to_scene = obs_data_get_string(request_data, "to_scene");
+	string transition;
+	auto duration = 0;
+	get_transition(from_scene, to_scene, transition, duration);
+	obs_data_set_string(response_data, "transition", transition.c_str());
+	obs_data_set_int(response_data, "duration", duration);
+	obs_data_set_bool(response_data, "success", true);
+}
+
+static void vendor_set_transition(obs_data_t *request_data,
+				  obs_data_t *response_data, void *param)
+{
+	UNUSED_PARAMETER(param);
+	std::string from_scene =
+		obs_data_get_string(request_data, "from_scene");
+	if (from_scene.empty()) {
+		obs_data_set_string(response_data, "error",
+				    "'from_scene' not set");
+		obs_data_set_bool(response_data, "success", false);
+		return;
+	}
+	std::string to_scene = obs_data_get_string(request_data, "to_scene");
+	if (to_scene.empty()) {
+		obs_data_set_string(response_data, "error",
+				    "'to_scene' not set");
+		obs_data_set_bool(response_data, "success", false);
+		return;
+	}
+	std::string transition =
+		obs_data_get_string(request_data, "transition");
+	if (transition.empty()) {
+		auto fs_it = transition_table.find(from_scene);
+		if (fs_it == transition_table.end()) {
+			obs_data_set_string(response_data, "error",
+					    "'from_scene' not found in table");
+			obs_data_set_bool(response_data, "success", false);
+			return;
+		}
+		auto ts_it = fs_it->second.find(to_scene);
+		if (ts_it == fs_it->second.end()) {
+			obs_data_set_string(
+				response_data, "error",
+				"'to_scene' not found for this 'from_scene'");
+			obs_data_set_bool(response_data, "success", false);
+			return;
+		}
+		fs_it->second.erase(ts_it);
+	} else {
+		int duration = obs_data_get_int(request_data, "duration");
+		auto &t = transition_table[from_scene][to_scene];
+		t.transition = transition;
+		t.duration = duration;
+	}
+	obs_data_set_bool(response_data, "success", true);
+}
+
+static void vendor_get_table(obs_data_t *request_data,
+			     obs_data_t *response_data, void *param)
+{
+	UNUSED_PARAMETER(request_data);
+	UNUSED_PARAMETER(param);
+	const auto transitions_array = obs_data_array_create();
+	for (const auto &it : transition_table) {
+		for (const auto &it2 : it.second) {
+			obs_data_t *transition = obs_data_create();
+			obs_data_set_string(transition, "from_scene",
+					    it.first.c_str());
+			obs_data_set_string(transition, "to_scene",
+					    it2.first.c_str());
+			obs_data_set_string(transition, "transition",
+					    it2.second.transition.c_str());
+			obs_data_set_int(transition, "duration",
+					 it2.second.duration);
+			obs_data_array_push_back(transitions_array, transition);
+			obs_data_release(transition);
+		}
+	}
+	obs_data_set_bool(response_data, "success", true);
+	obs_data_set_array(response_data, "transitions", transitions_array);
+	obs_data_array_release(transitions_array);
+}
+
+void obs_module_post_load(void)
+{
+	vendor = obs_websocket_register_vendor("transition-table");
+	if (!vendor)
+		return;
+	obs_websocket_vendor_register_request(vendor, "get_transition",
+					      vendor_get_transition, nullptr);
+	obs_websocket_vendor_register_request(vendor, "set_transition",
+					      vendor_set_transition, nullptr);
+	obs_websocket_vendor_register_request(vendor, "get_table",
+					      vendor_get_table, nullptr);
+}
+
 void obs_module_unload(void)
 {
 	obs_hotkey_pair_unregister(transition_table_hotkey);
